// WideMargin. Simple fast bible software.
// Copyright (C) 2011  Daniel Hughes
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using WideMargin.MVCInterfaces;
using WideMargin.Database;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using WideMargin.Utilities;
using SharpSword;

namespace WideMargin.Controller
{
	/// <summary>
	/// Controller for the books view, used for mouse friendly lookup of passages.
	/// </summary>
	public class BooksController
	{
		Action<Action> _changeView;
		IBibleView _bibleView;
		IFilteredView _booksView;
		BibleDatabase _bible;
		
		/// <summary>
		/// Initializes a new instance of the <see cref="WideMargin.Controller.BooksController"/> class.
		/// </summary>
		/// <param name='changeView'>Changes the view in a way which means the history works</param>
		/// <param name='bibleView'>Access to a tab in widemargin.</param>
		public BooksController (Action<Action> changeView, IBibleView bibleView, BibleDatabase bible)
		{
			_changeView = changeView;
			_bibleView = bibleView;
			_booksView = _bibleView.BooksView;
			_bible = bible;
			
			var bookList = BookList.Get();
			_booksView = _bibleView.BooksView;
			_booksView.ActionLinkClicked += HandleBooksViewActionLinkClicked;
			_booksView.VerseLinkClicked += HandleVerseLinkClicked;
			
			_booksView.Filters = new Filter[]
			{
				new Filter("All", () => ShowBookList(bookList.AllBookNames())),
				new Filter("Old Testiment", () => ShowBookList(bookList.GetCatagory("Old Testiment"))),
				new Filter("New Testiment", () => ShowBookList(bookList.GetCatagory("New Testiment"))),
				new Filter("Torah", () => ShowBookList(bookList.GetCatagory("Torah"))),
				new Filter("History", () => ShowBookList(bookList.GetCatagory("History"))),
				new Filter("Poetry", () => ShowBookList(bookList.GetCatagory("Poetry"))),
				new Filter("Prophecy", () => ShowBookList(bookList.GetCatagory("Prophecy"))),
				new Filter("Gospels", () => ShowBookList(bookList.GetCatagory("Gospels"))),
				new Filter("Letters", () => ShowBookList(bookList.GetCatagory("Letters"))),
			};
		}
		
		/// <summary>
		/// A verse link has been clicked. Pass it on to the bible controller
		/// </summary>
		private void HandleVerseLinkClicked (object sender, LinkClickedEventArgs<IVerseIdentifier> e)
		{
			if(VerseLinkClicked != null)
			{
				VerseLinkClicked(this, e);	
			}
		}
		
		/// <summary>
		/// Occurs when a verse link is clicked.
		/// </summary>
		public event EventHandler<LinkClickedEventArgs<IVerseIdentifier>> VerseLinkClicked;
		
		/// <summary>
		/// Occurs when a book link is clicked.
		/// </summary>
		public event EventHandler<LinkClickedEventArgs<string>> BookLinkClicked;
			
		/// <summary>
		/// Handles the books view action link clicked. 
		/// Which occurs when a book is clicked.
		/// Generates a list of chapters with verse links.
		/// </summary>
		void HandleBooksViewActionLinkClicked (object sender, LinkClickedEventArgs<string> e)
		{
			string target = e.LinkTarget;
			if(e.LinkTarget.StartsWith("Chapters/"))
			{
				target = e.LinkTarget.Split('/')[1];
			}
			
			if(e.Button == 1)
			{
				ShowChapters(target);
				return;
			}
			if(e.Button == 2)
			{
				BookLinkClicked.Fire(this, new LinkClickedEventArgs<string>(target, 2));
				return;
			}
		}
		
		/// <summary>
		/// Shows the chapters for a book
		/// </summary>
		/// <param name='book'>book</param>
		public void ShowChapters(string book)
		{
			var chapterIdentifiers = _bible.GetVersification().GetChapters(book);
			
			var chapters = new Chapters();
			var chaptersHtml = chapters.GetChaptersHtml(chapterIdentifiers);
			
			_changeView(() =>
			{
				_booksView.Text = chaptersHtml;
				_bibleView.ShowBooksView();
				_bibleView.ChangeTitle("Browse");
			});
		}
		
		/// <summary>
		/// Shows a list of books.
		/// </summary>
		/// <param name='bookList'>books to show</param>
		public void ShowBookList(IEnumerable<string> bookList)
	    {
			var booksMarkup = new Books();
			var booksHtml = booksMarkup.GetBooksHtml(bookList);
			_changeView(() =>
			{
				_booksView.Text = booksHtml;
				_bibleView.ShowBooksView();
				_bibleView.ChangeTitle("Browse");
			});
		}
	}
}

