// WideMargin. Simple fast bible software.
// Copyright (C) 2011-2013  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Threading;
using Gtk;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.GUI
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class SearchPage : Gtk.Bin, IBibleView
	{
		Label _tabLabel;
		Widget _content;
		FilteredView _searchResultsWidget;
		PassagePage _passagePage;
		GenericView _genericView;
		FilteredView _booksView;
		Gdk.Pixbuf _closeImage;
		ListStore _completionStore;
		bool _completionInitalised = false;
		Frame _frame;
		bool _editible;
		EntryCompletion _completion;
		
		public SearchPage ()
		{
			this.Build ();
			
			_bibleBar.Changed += BibleBarChanged;
			_passagePage = new PassagePage();
			_searchResultsWidget = new FilteredView();
			_genericView = new GenericView();
			_booksView = new FilteredView();
			
			_closeImage = ImageCache.Get().CloseImage;
			this.CanFocus = true;
			_completion = new EntryCompletion();
			_completion.TextColumn = 0;
			_completion.MatchFunc = CompletionMatch;
			_completionStore = new ListStore (typeof (string));
			
			_completion.Model = _completionStore;
			_bibleBar.Completion = _completion;
			
			_backButton.Image = new Image(ImageCache.Get().BackArrow);
			_forwardButton.Image = new Image(ImageCache.Get().ForwardArrow);
			_menuButton.Image = new Image(ImageCache.Get().Settings);
			_browseButton.Image = new Image(ImageCache.Get().Book);
			_frame = new Frame();
			_frame.ShadowType = ShadowType.EtchedIn;
			vbox4.Add(_frame);
		}
		
		public event EventHandler<EventArgs> Browse;
		
		bool CompletionMatch(EntryCompletion completion, string key, TreeIter iter)
		{
			var valueToMatch = (string)_completionStore.GetValue(iter, 0);
			valueToMatch = valueToMatch.ToLower();
			key = key.ToLower();
			if(valueToMatch.StartsWith(key) && valueToMatch != key)
			{
				return true;
			}
			return false;
		}
		
		public bool Editable
		{
			get
			{
				return _editible;	
			}
			set
			{
				_editible = value;
				_bibleBar.IsEditable = _editible;
			}
		}
	
			
		public void FocusBibleBar()
		{
			ThreadPool.QueueUserWorkItem(FocusBibleBarCallback);
		}
			                                              
		public void FocusBibleBarCallback(object state)
		{
			this.Invoke(() => _bibleBar.GrabFocus());
		}
		
		/// <summary>
		/// Tiggered to Request about info to show in
		/// the about box.
		/// </summary>
		public event EventHandler<EventArgs> RequestAboutInfo;
		
		public event EventHandler<EventArgs> Closing;
		
		public event EventHandler<EventArgs> RequestAutoCompleteList;
		
		public Widget CreateTabTitle()
		{
			HBox box = new HBox(false, 0);
			box.BorderWidth = 0;
			
			//add tab label
			_tabLabel = new Label("New Tab");
			_tabLabel.ClearFlag(WidgetFlags.CanFocus);
			_tabLabel.SetPadding(0,0);
			box.PackStart(_tabLabel,true,true,0);
			
			//add close button
			Button button = new Button();
			Gtk.Image closeIcon = new Gtk.Image (_closeImage);
			closeIcon.SetPadding (0, 0);
			button.Image = closeIcon;
			button.Clicked += OnCloseClicked;
			button.Relief = ReliefStyle.None;
			button.CanDefault = false;
			button.BorderWidth = 0;
			box.PackEnd(button,false,true,0);
			
			box.ClearFlag (WidgetFlags.CanFocus);
			box.CanFocus = false;
			box.ShowAll();
			return box;
		}
		
		void OnCloseClicked(object sender, EventArgs args)
		{
			Closing.Fire(this, args);
		}
		
		protected virtual void BibleBarActivated (object sender, System.EventArgs e)
		{
			BibleBarChanged(sender, e);
		}
	
		protected virtual void BibleBarChanged (object sender, System.EventArgs e)
		{
			if(!_completionInitalised)
			{
				RequestAutoCompleteList.Fire(this, new EventArgs());
				_completionInitalised = true;
			}
			
			SearchRequest.Fire(this, new StringEventArgs(_bibleBar.Text));
		}
		
		void ShowWidget(Widget widget)
		{
			
			if(_content != null && widget != _content)
			{
				_frame.Remove(_content);
			}
			if(widget != _content)
			{
				_frame.Add(widget);
				_content = widget;
				vbox4.ShowAll();
			}
		}
		
		public event EventHandler<StringEventArgs> SearchRequest;
		
		/// <summary>
		/// Occurs when the user requests to navigate backwards
		/// </summary>
		public event EventHandler<EventArgs> NavigateBackwards;
		
		/// <summary>
		/// Occurs when the user requests to navigate forwards.
		/// </summary>
		public event EventHandler<EventArgs> NavigateForwards;
		
		public IPassageView PassageView
		{
			get
			{
				return _passagePage;	
			}
		}
		
		public IFilteredView SearchView
		{
			get
			{
				return _searchResultsWidget;	
			}
		}
		
		public IGenericView GenericView
		{
			get
			{
				return _genericView;	
			}
		}
		
		public IFilteredView BooksView
		{
			get
			{
				return _booksView;	
			}
		}
		
		public void ShowPassageView()
		{
			ThreadSafeShow(_passagePage);
		}
		
		public void ShowSearchView()
		{
			ThreadSafeShow(_searchResultsWidget);
		}
		
		public void ShowBooksView()
		{
			ThreadSafeShow(_booksView);	
		}
		
		/// <summary>
		/// Shows the GenericView, this is used for the dayly readings.
		/// </summary>
		public void ShowGenericView()
		{
			ThreadSafeShow(_genericView);	
		}
		
		public void ThreadSafeShow(Widget widget)
		{			
			this.Invoke(() =>
			{
				ShowWidget(widget);
			});
		}
		
		public void ChangeTitle(string title)
		{
			this.Invoke(() =>
			{
				_tabLabel.Text = title;
			});
		}
		
		public void ChangeBibleBar(string text)
		{
			this.Invoke(() =>
			{
				_bibleBar.Text = text;
			});
		}
		
		public void ShowAutoCompleteList(IEnumerable<string> list)
		{
			this.Invoke(() =>
			{
				foreach(string book in list)
				{
					_completionStore.AppendValues (book);
				}
			});
		}
		
		/// <summary>
		/// Occurs when the back button is clicked.
		/// Rises the event to the controller so it 
		/// can navigate backwards
		/// </summary>
		protected virtual void HandleForwardButtonClicked (object sender, EventArgs e)
		{
			NavigateForwards.Fire(this, EventArgs.Empty);
		}
		
		/// <summary>
		/// Occurs when the forward button is clicked.
		/// Raises the event to the controller so it 
		/// can navigate forwards.
		/// </summary>
		protected virtual void HandleBackButtonClicked (object sender, EventArgs e)
		{
			NavigateBackwards.Fire(this, EventArgs.Empty);
		}
		
		/// <summary>
		/// User has clicked on the settings menu button.
		/// Shows the menu
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected virtual void HandleMenuClicked (object sender, System.EventArgs e)
		{
			Menu menu = new Menu();
			MenuItem aboutMenuItem = new MenuItem("About");
			aboutMenuItem.Activated += HandleAboutMenuItemActivated;
			menu.Add(aboutMenuItem);
			
			menu.ShowAll();
			menu.Popup(null, null, MenuPosition, 0, 0);
		}

		/// <summary>
		/// The About menu item has been activated
		/// Passes this request on to the controller.
		/// </summary>
		/// <param name="sender">about menu item</param>
		/// <param name="args">default event args</param>
		void HandleAboutMenuItemActivated (object sender, EventArgs args)
		{
			RequestAboutInfo.Fire(this, args);
		}
		
		/// <summary>
		/// Callback for the setting the position of the settings menu
		/// </summary>
		/// <param name="menu">the menu to set the position for</param>
		/// <param name="x">out param for the x cordinate</param>
		/// <param name="y">out param for the y cordinate</param>
		/// <param name="pushIn">out param for weather to push it onto the screen if drawn off it.</param>
		void MenuPosition(Menu menu, out int x, out int y, out bool pushIn)
	    {
			_menuButton.ParentWindow.GetOrigin(out x, out y);
			int labelWidth = menu.SizeRequest().Width; //need to look up actual width.
			x += _menuButton.Allocation.X - labelWidth + _menuButton.Allocation.Width;
			y += _menuButton.Allocation.Y + _menuButton.Allocation.Height;
			pushIn = true;
		}

		/// <summary>
		/// Runs the action on the UI thread
		/// </summary>
		/// <param name='action'>
		/// Action to run.
		/// </param>
		public void RunOnUiThread(System.Action action)
		{
			this.Invoke(action);
		}
		
		protected void HandleBrowseButtonClicked (object sender, EventArgs e)
		{
			Browse.Fire(this, EventArgs.Empty);
		}
	}
}

