﻿using SharpSword;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;
using System.Xml.Linq;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
    /// <summary>
    /// Interaction logic for RichTextView.xaml
    /// </summary>
    public partial class WebView : UserControl
    {
        ManualResetEvent _waitForLoad;
        volatile bool _loading = false;

        public WebView()
        {
            InitializeComponent();
            _browserControl.Navigating += BrowserControl_Navigating;
            _browserControl.LoadCompleted += BrowserControlLoadCompleted;
            _waitForLoad = new ManualResetEvent(true);

            int feature = FEATURE_DISABLE_NAVIGATION_SOUNDS;
            CoInternetSetFeatureEnabled(feature, SET_FEATURE_ON_PROCESS, true);
        }

        void BrowserControlLoadCompleted(object sender, NavigationEventArgs e)
        {
            _loading = false;
            _waitForLoad.Set();
            if (!IsScrollbarVisible())
            {
                RequestMore.Fire(this, EventArgs.Empty);
            }
        }

        const int FEATURE_DISABLE_NAVIGATION_SOUNDS = 21;
        const int SET_FEATURE_ON_PROCESS = 0x00000002;

        [DllImport("urlmon.dll")]
        [PreserveSig]
        [return: MarshalAs(UnmanagedType.Error)]
        static extern int CoInternetSetFeatureEnabled(int FeatureEntry, [MarshalAs(UnmanagedType.U4)] int dwFlags, bool fEnable);

        bool IsScrollbarVisible()
        {
            var document = (mshtml.IHTMLDocument2)_browserControl.Document;
            int totalHeight = document.body.offsetHeight;

            if (totalHeight <= this._browserControl.ActualHeight)
            {
                //not shown
                return false;
            }
            return true;
        }

        bool ShouldIncludedElement(XElement element)
        {
            if (element.Attributes("class").Any())
            {
                string value =  element.Attributes("class").First().Value;
                if (value == "note" || value == "notelink")
                {
                    return false;
                }
            }
            return true;
        }

        bool ShouldAppendLine(XElement element)
        {
            if (element.Name == "BR")
            {
                return true;
            }
            if (element.Attributes("class").Any())
            {
                string value = element.Attributes("class").First().Value;
                if (value == "line" || value == "firstLine")
                {
                    return true;
                }
            }
            if (element.Attributes("id").Any())
            {
                string value = element.Attributes("id").First().Value;
                if (value == "heading")
                {
                    return true;
                }
            }
            return false;
        }

        bool IsVerseNumber(XElement element)
        {
            if (element.Attributes("class").Any())
            {
                string value = element.Attributes("class").First().Value;
                if (value == "verseNumber")
                {
                    return true;
                }
            }
            return false;
        }

        void ParseHtmlToPlainText(XElement element, StringBuilder builder)
        {
            foreach (XNode node in element.Nodes())
            {
                if (node.NodeType == XmlNodeType.Element)
                {
                    var child = node as XElement;
                    if (ShouldIncludedElement(child))
                    {
                        if (IsVerseNumber(child))
                        {
                            builder.Append(" ");
                        }
                        ParseHtmlToPlainText(child, builder);
                        if (IsVerseNumber(child))
                        {
                            builder.Append(". ");
                        }
                        if (ShouldAppendLine(child))
                            builder.AppendLine();
                    }
                }
                else if (node.NodeType == XmlNodeType.Text)
                {
                    builder.Append(node.ToString());
                }
            }
        }

        void BrowserControl_Navigating(object sender, NavigatingCancelEventArgs e)
        {
            e.Cancel = true;
            if (e.Uri == null)
            {
                e.Cancel = false;
                return;
            }
            var target = e.Uri.ToString();

            //get versification
            var requestVersificationEventArgs = new RequestVersificationEventArgs();
            RequestVersification.Fire(this, requestVersificationEventArgs);
            var versification = requestVersificationEventArgs.Versification;

            target = target.Replace("about:", string.Empty);

            if (target == "Copy")
            {
                //strip formatting from clipboard.
                new Task(() =>
                {
                    Dispatcher.Invoke((Action)(() => 
                        {
                            var text = (string)Clipboard.GetData(DataFormats.Html);
                            int startIndex = text.IndexOf("<BODY");
                            int endIndex = text.IndexOf("</BODY>") + 7;
                            text = text.Substring(startIndex, endIndex - startIndex);
                            text = text.Replace("<BR>", "<BR />");
                            var element = XElement.Parse(text);
                            var plainText = new StringBuilder();
                            ParseHtmlToPlainText(element, plainText);
                            text = text.Replace(Environment.NewLine, "");
                            Clipboard.SetText(plainText.ToString());
                        }));
                }).Start();
                return;
            }

            if (target == "RequestMore")
            {
                e.Cancel = true;
                this.RequestMore.Fire(this, EventArgs.Empty);
                return;
            }

            var targetParts = target.Split(',');
            if (targetParts.Length < 2)
            {
                e.Cancel = false;
                return;
            }
            int mouseButton = int.Parse(targetParts[1]) + 1;


            IVerseIdentifier verseTarget;
            if (versification.TryCreateVerseIdentifier(targetParts[0], out verseTarget))
            {
                e.Cancel = true;
                LinkClickedEventArgs<IVerseIdentifier> linkClickedEventArgs = new LinkClickedEventArgs<IVerseIdentifier>(verseTarget, MaIEMouseButtons(mouseButton));
                new Task(() =>
                {
                    VerseLinkClicked.Fire(this, linkClickedEventArgs); ;
                }).Start();
                return;
            }
            e.Cancel = true;
            new Task(() =>
            {
                ActionLinkClicked.Fire(this, new LinkClickedEventArgs<string>(targetParts[0], MaIEMouseButtons(mouseButton)));
            }).Start();
        }

        int MaIEMouseButtons(int button)
        {
            if (button == 5)
                return 2;
            if (button == 2)
                return 1;
            return button;
        }

        public void AppendText(string text)
        {
            text = text.Replace(Environment.NewLine, string.Empty).Replace("'", "&#39;");
            ExecuteScript("AppendMarkup", text);
        }

        public event EventHandler<RequestVersificationEventArgs> RequestVersification;

        /// <summary>
        /// Occurs when a verse link is clicked.
        /// </summary>
        public event EventHandler<LinkClickedEventArgs<IVerseIdentifier>> VerseLinkClicked;

        /// <summary>
        /// Occurs when a action link is clicked.
        /// </summary>
        public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;

        public event EventHandler<EventArgs> RequestMore;

        static readonly int InstalledIEVersion = new System.Windows.Forms.WebBrowser().Version.Major;
        static readonly int IEVersion = InstalledIEVersion > 10 ? 10 : InstalledIEVersion;
        static readonly string IECompatibleHeader = "<head><meta http-equiv=\"X-UA-Compatible\" content=\"IE=" + IEVersion.ToString() + "\"><meta charset=\"utf-8\">";

        internal string Text
        {
            set
            {
                _loading = true;
                _waitForLoad.Reset();
                //<meta http-equiv="X-UA-Compatible" content="IE=10">
                var html = value.Replace("<head>", IECompatibleHeader);
                _browserControl.NavigateToString(html);
            }
        }

        internal void ExecuteScript(string script, params object[] args)
        {
            if (_loading)
            {
                new Task(() =>
                {
                    _waitForLoad.WaitOne();
                    Dispatcher.Invoke((Action)(() => 
                        {
                            _browserControl.InvokeScript(script, args);
                        }));
                }).Start();
                return;
            }

            _browserControl.InvokeScript(script, args);
        }
    }
}
